import axios from 'axios'
import toCamel from 'camelcase-keys'
import { useEffect, useRef, useState } from 'react'
import { Helmet } from 'react-helmet-async'
import { Link, useLocation } from 'react-router-dom'

import TagSidebar from '@/components/TagSidebar'
import WikiBody from '@/components/WikiBody'
import TabGroup, { Tab } from '@/components/common/TabGroup'
import MainArea from '@/components/layout/MainArea'
import { API_BASE_URL, SITE_TITLE } from '@/config'

import type { Post, WikiPage } from '@/types'


export default () => {
  const [posts, setPosts] = useState<Post[]> ([])
  const [wikiPage, setWikiPage] = useState<WikiPage | null> (null)
  const [cursor, setCursor] = useState ('')
  const [loading, setLoading] = useState (false)

  const loaderRef = useRef<HTMLDivElement | null> (null)

  const loadMore = async (withCursor: boolean) => {
    setLoading (true)
    const res = await axios.get (`${ API_BASE_URL }/posts`, {
	params: { tags:   tags.join (' '),
		  match:  (anyFlg ? 'any' : 'all'),
		  ...(withCursor ? { cursor } : { }) } })
    const data = toCamel (res.data as any, { deep: true }) as { posts:      Post[]
								nextCursor: string }
    setPosts (posts => [...(withCursor ? posts : []), ...data.posts])
    setCursor (data.nextCursor)
    setLoading (false)
  }

  const location = useLocation ()
  const query = new URLSearchParams (location.search)
  const tagsQuery = query.get ('tags') ?? ''
  const anyFlg = query.get ('match') === 'any'
  const tags = tagsQuery.split (' ').filter (e => e !== '')

  useEffect(() => {
    const observer = new IntersectionObserver (entries => {
      if (entries[0].isIntersecting && !(loading) && cursor)
	loadMore (true)
    }, { threshold: 1 })

    const target = loaderRef.current
    target && observer.observe (target)

    return () => {
      target && observer.unobserve (target)
    }
  }, [loaderRef, loading])

  useEffect (() => {
    setPosts ([])
    loadMore (false)

    setWikiPage (null)
    if (tags.length === 1)
      {
	void (async () => {
	  try
	  {
	    const tagName = tags[0]
	    const res = await axios.get (`${ API_BASE_URL }/wiki/title/${ tagName }`)
	    setWikiPage (toCamel (res.data as any, { deep: true }) as WikiPage)
	  }
	  catch
	  {
	    ;
	  }
	}) ()
      }
  }, [location.search])

  return (
      <>
	<Helmet>
	  <title>
	    {tags.length
	     ? `${ tags.join (anyFlg ? ' or ' : ' and ') } | ${ SITE_TITLE }`
	     : `${ SITE_TITLE }　〜 ぼざろクリーチャーシリーズ綜合リンク集サイト`}
	  </title>
	</Helmet>
	<TagSidebar posts={posts.slice (0, 20)} />
	<MainArea>
	  <TabGroup>
	    <Tab name="広場">
	      {posts.length
	       ? (
		  <div className="flex flex-wrap gap-4 p-4">
		  {posts.map (post => (
		      <Link to={`/posts/${ post.id }`}
			    key={post.id}
			    className="w-40 h-40 overflow-hidden rounded-lg shadow-md hover:shadow-lg">
			<img src={post.thumbnail || post.thumbnailBase || undefined}
			     alt={post.title || post.url}
			     title={post.title || post.url || undefined}
			     className="object-none w-full h-full" />
		      </Link>))}
		  </div>)
	       : !(loading) && '広場には何もありませんよ．'}
	      {loading && 'Loading...'}
	      <div ref={loaderRef} className="h-12"></div>
	    </Tab>
	    {(wikiPage && wikiPage.body) && (
		<Tab name="Wiki" init={!(posts.length)}>
		  <WikiBody body={wikiPage.body} />
		  <div className="my-2">
		    <Link to={`/wiki/${ wikiPage.title }`}>Wiki を見る</Link>
		  </div>
		</Tab>)}
	  </TabGroup>
	</MainArea>
      </>)
}
